@@ -27,6 +27,31 @@
 #include "app-config.h"
 #include "main-win.h"
 #include "tab-page.h"
+#include "pcmanfm.h"
+
+/* Additional entries for FmFileMenu popup */
+static const char folder_menu_xml[]=
+"<popup>"
+  "<placeholder name='ph1'>"
+    "<menuitem action='NewTab'/>"
+    "<menuitem action='NewWin'/>"
+    "<menuitem action='Term'/>"
+    /* "<menuitem action='Search'/>" */
+  "</placeholder>"
+"</popup>";
+
+static void on_open_in_new_tab(GtkAction* act, FmMainWin* win);
+static void on_open_in_new_win(GtkAction* act, FmMainWin* win);
+static void on_open_folder_in_terminal(GtkAction* act, FmMainWin* win);
+
+/* Action entries for popup menu entries above */
+static GtkActionEntry folder_menu_actions[]=
+{
+    {"NewTab", GTK_STOCK_NEW, N_("Open in New T_ab"), NULL, NULL, G_CALLBACK(on_open_in_new_tab)},
+    {"NewWin", GTK_STOCK_NEW, N_("Open in New Win_dow"), NULL, NULL, G_CALLBACK(on_open_in_new_win)},
+    {"Search", GTK_STOCK_FIND, NULL, NULL, NULL, NULL},
+    {"Term", "utilities-terminal", N_("Open in Termina_l"), NULL, NULL, G_CALLBACK(on_open_folder_in_terminal)},
+};
 
 #define GET_MAIN_WIN(page)   FM_MAIN_WIN(gtk_widget_get_toplevel(GTK_WIDGET(page)))
 
@@ -245,10 +270,27 @@ static FmJobErrorAction on_folder_error(FmFolder* folder, GError* err, FmJobErro
 
 static void on_folder_start_loading(FmFolder* folder, FmTabPage* page)
 {
+    FmFolderView* fv = page->folder_view;
     /* g_debug("start-loading"); */
     /* FIXME: this should be set on toplevel parent */
     fm_set_busy_cursor(GTK_WIDGET(page));
-    fm_folder_view_set_model(page->folder_view, NULL);
+
+#if FM_CHECK_VERSION(1, 0, 2)
+    if(fm_folder_is_incremental(folder))
+    {
+        /* create a model for the folder and set it to the view
+           it is delayed for non-incremental folders since adding rows into
+           model is much faster without handlers connected to its signals */
+        FmFolderModel* model = fm_folder_model_new(folder, FALSE);
+        fm_folder_view_set_model(fv, model);
+        fm_folder_model_set_sort(model, app_config->sort_by,
+                                 (app_config->sort_type == GTK_SORT_ASCENDING) ?
+                                        FM_SORT_ASCENDING : FM_SORT_DESCENDING);
+        g_object_unref(model);
+    }
+    else
+#endif
+        fm_folder_view_set_model(fv, NULL);
 }
 
 static void on_folder_finish_loading(FmFolder* folder, FmTabPage* page)
@@ -257,10 +299,26 @@ static void on_folder_finish_loading(FmFolder* folder, FmTabPage* page)
     const FmNavHistoryItem* item;
     GtkScrolledWindow* scroll = GTK_SCROLLED_WINDOW(fv);
 
-    /* create a model for the folder and set it to the view */
-    FmFolderModel* model = fm_folder_model_new(folder, app_config->show_hidden);
-    fm_folder_view_set_model(fv, model);
-    g_object_unref(model);
+    /* Note: most of the time, we delay the creation of the 
+     * folder model and do it after the whole folder is loaded.
+     * That is because adding rows into model is much faster when no handlers
+     * are connected to its signals. So we detach the model from folder view
+     * and create the model again when it's fully loaded. 
+     * This optimization, however, is not used for FmFolder objects
+     * with incremental loading (search://) */
+    if(fm_folder_view_get_model(fv) == NULL)
+    {
+        /* create a model for the folder and set it to the view */
+        FmFolderModel* model = fm_folder_model_new(folder, app_config->show_hidden);
+        fm_folder_view_set_model(fv, model);
+#if FM_CHECK_VERSION(1, 0, 2)
+        /* since 1.0.2 sorting should be applied on model instead of view */
+        fm_folder_model_set_sort(model, app_config->sort_by,
+                                 (app_config->sort_type == GTK_SORT_ASCENDING) ?
+                                        FM_SORT_ASCENDING : FM_SORT_DESCENDING);
+#endif
+        g_object_unref(model);
+    }
     fm_folder_query_filesystem_info(folder); /* FIXME: is this needed? */
 
     // fm_path_entry_set_path(entry, path);
@@ -276,7 +334,7 @@ static void on_folder_finish_loading(FmFolder* folder, FmTabPage* page)
                   (guint)FM_STATUS_TEXT_NORMAL,
                   page->status_text[FM_STATUS_TEXT_NORMAL]);
 
-    fm_unset_busy_cursor(GTK_WIDGET(fv));
+    fm_unset_busy_cursor(GTK_WIDGET(page));
     /* g_debug("finish-loading"); */
 }
 
@@ -333,22 +391,92 @@ static char* format_status_text(FmTabPage* page)
     return NULL;
 }
 
+static void on_open_in_new_tab(GtkAction* act, FmMainWin* win)
+{
+    FmPathList* sels = fm_folder_view_dup_selected_file_paths(win->folder_view);
+    GList* l;
+    for( l = fm_path_list_peek_head_link(sels); l; l=l->next )
+    {
+        FmPath* path = (FmPath*)l->data;
+        fm_main_win_add_tab(win, path);
+    }
+    fm_path_list_unref(sels);
+}
+
+static void on_open_in_new_win(GtkAction* act, FmMainWin* win)
+{
+    FmPathList* sels = fm_folder_view_dup_selected_file_paths(win->folder_view);
+    GList* l;
+    for( l = fm_path_list_peek_head_link(sels); l; l=l->next )
+    {
+        FmPath* path = (FmPath*)l->data;
+        fm_main_win_add_win(win, path);
+    }
+    fm_path_list_unref(sels);
+}
+
+static void on_open_folder_in_terminal(GtkAction* act, FmMainWin* win)
+{
+    FmFileInfoList* files = fm_folder_view_dup_selected_files(win->folder_view);
+    GList* l;
+    for(l=fm_file_info_list_peek_head_link(files);l;l=l->next)
+    {
+        FmFileInfo* fi = (FmFileInfo*)l->data;
+        if(fm_file_info_is_dir(fi) /*&& !fm_file_info_is_virtual(fi)*/)
+            pcmanfm_open_folder_in_terminal(GTK_WINDOW(win), fm_file_info_get_path(fi));
+    }
+    fm_file_info_list_unref(files);
+}
+
+/* folder view popups */
+static void update_files_popup(FmFolderView* fv, GtkWindow* win,
+                               GtkUIManager* ui, GtkActionGroup* act_grp,
+                               FmFileInfoList* files)
+{
+    gtk_action_group_set_translation_domain(act_grp, NULL);
+    gtk_action_group_add_actions(act_grp, folder_menu_actions,
+                                 G_N_ELEMENTS(folder_menu_actions), win);
+    gtk_ui_manager_add_ui_from_string(ui, folder_menu_xml, -1, NULL);
+}
+
+static gboolean open_folder_func(GAppLaunchContext* ctx, GList* folder_infos, gpointer user_data, GError** err)
+{
+    FmMainWin* win = FM_MAIN_WIN(user_data);
+    GList* l = folder_infos;
+    FmFileInfo* fi = (FmFileInfo*)l->data;
+    fm_main_win_chdir(win, fm_file_info_get_path(fi));
+    l=l->next;
+    for(; l; l=l->next)
+    {
+        FmFileInfo* fi = (FmFileInfo*)l->data;
+        fm_main_win_add_tab(win, fm_file_info_get_path(fi));
+    }
+    return TRUE;
+}
+
 static void fm_tab_page_init(FmTabPage *page)
 {
     GtkPaned* paned = GTK_PANED(page);
     FmTabLabel* tab_label;
     FmFolderView* folder_view;
     GList* focus_chain = NULL;
+    FmSidePaneMode mode = app_config->side_pane_mode;
 
     page->side_pane = fm_side_pane_new();
-    fm_side_pane_set_mode(page->side_pane, app_config->side_pane_mode);
+    fm_side_pane_set_mode(page->side_pane, (mode & FM_SP_MODE_MASK));
     /* TODO: add a close button to side pane */
     gtk_paned_add1(paned, GTK_WIDGET(page->side_pane));
     focus_chain = g_list_prepend(focus_chain, page->side_pane);
 
-    folder_view = fm_folder_view_new(app_config->view_mode);
+    /* handlers below will be used when FmMainWin detects new page added */
+    folder_view = (FmFolderView*)fm_standard_view_new(app_config->view_mode,
+                                                      update_files_popup,
+                                                      open_folder_func);
     page->folder_view = folder_view;
+#if !FM_CHECK_VERSION(1, 0, 2)
+    /* since 1.0.2 sorting should be applied on model instead */
     fm_folder_view_sort(folder_view, app_config->sort_type, app_config->sort_by);
+#endif
     fm_folder_view_set_selection_mode(folder_view, GTK_SELECTION_MULTIPLE);
     page->nav_history = fm_nav_history_new();
     gtk_paned_add2(paned, GTK_WIDGET(page->folder_view));
@@ -359,6 +487,8 @@ static void fm_tab_page_init(FmTabPage *page)
     g_list_free(focus_chain);
 
     gtk_widget_show_all(GTK_WIDGET(page));
+    if(mode & FM_SP_HIDE)
+        gtk_widget_hide(GTK_WIDGET(page->side_pane));
 
     /* create tab label */
     tab_label = (FmTabLabel*)fm_tab_label_new("");
@@ -411,6 +541,7 @@ static void fm_tab_page_chdir_without_history(FmTabPage* page, FmPath* path)
 
     if(fm_folder_is_loaded(page->folder))
     {
+        on_folder_start_loading(page->folder, page);
         on_folder_finish_loading(page->folder, page);
         on_folder_fs_info(page->folder, page);
     }
