@@ -68,6 +68,9 @@ struct _FmTabPageClass
     void (*status)(FmTabPage* page, guint type, const char* status_text);
 };
 
+/* Side Pane mode extensions */
+#define FM_SP_MODE_MASK 0x1f
+#define FM_SP_HIDE      0x20
 
 GType fm_tab_page_get_type(void);
 
